import java.util.Random;

/** Classe que contm todas as informaes de um cromossomo, tais como
 * fitness, chance de reproduo e rota (possvel soluo para o problema)
 * <BR><BR>
 * Trabalho de Algoritmos Genticos <BR> 
 * Disciplina de Inteligncia Artificial <BR>
 * Sistemas de Informao - 01/2009 <BR>
 * Professor: Guilherme Dhein <BR>
 * <BR>
 * @author Douglas Pasqualin
 * @author Reinaldo Dornelles
 *
 */
public class Cromossomo
{
	public int[] individuo;
	private int totalCidades;
	private int fitness;
	private double chanceReproducao;
	private Custos custos;
	
	/**
	 * Construtor sem parmetros somente instancia a classe.
	 */	
	public Cromossomo()
	{
		this.fitness = 0;
	}

	/**
	 * Mtodo construtor. Cria uma rota aleatria iniciando sempre na cidade zero e 
	 * j calcula o fitness do cromossomo.
	 * @param totalCidades total de cidades que estaro na rota para o caixeiro viajar
	 * @param custos objeto do tipo Custos com os custos de deslocamento de todas as cidades
	 */		
	public Cromossomo(int totalCidades, Custos custos) 
	{
		this.totalCidades = totalCidades;
		this.custos = custos;
		this.fitness = 0;
		individuo = new int[totalCidades-1];
		geraCromossomo();
		calculaFitness();
	}
	
	/**
	 * Mtodo que gera aleatoriamente as cidades onde o caixeiro dever visitar. <BR>
	 * Sempre inicia na cidade zero.
	 */	
	private void geraCromossomo()
	{
		Random random = new Random();
		int x;
		for (int i=0; i< totalCidades-1; i++)
		{
			do 
			{
				x = random.nextInt(totalCidades);	
			} while (cidadeJaEstaNaRota(x));
			individuo[i] = x;
		}
	}

	/**
	 * Mtodo que verifica se uma cidade j est na rota do caixeiro. <BR>
	 * Necessrio pois o caixeiro s deve visitar a cidade um vez, no pode haver repetio.
	 * @param cidade cidade a ser testada 
	 * @return <b>true</b> Se a cidade j est na roda e <b>false</b> se a cidade no est na rota
	 */	
	private boolean cidadeJaEstaNaRota(int cidade)
	{
		if (cidade == 0) return true;
		for (int i=0; i< totalCidades-1; i++)
			if (individuo[i] == cidade)
				return true;
		return false;
	}

	/**
	 * Mtodo que calcula o fitness (aptido) de um indviduo baseado
	 * no custo de distncia entre uma cidade e outra. <BR>
	 * Quanto menor o fitness maior deve ser a adptido do indviduo
	 */		
	private void calculaFitness()
	{
		this.fitness = custos.getCusto(0, individuo[0]);
		for (int i=0; i< totalCidades-2; i++)
			this.fitness += custos.getCusto(individuo[i], individuo[i+1]);
		//da ltima cidade para a primeira (voltar)
		this.fitness += custos.getCusto(individuo[individuo.length-1], 0);
	}

	/**
	 * Mtodo auxiliar para mostrar todas as cidades da rota
	 * @return String no formato cidade1 >> cidade2 >> cidade3 .....
	 */		
	public String individuoToString()
	{
		String resultado = "0 >> ";
		for (int i=0; i< totalCidades-1; i++)
			resultado += individuo[i] + " >> ";
		resultado += 0;
		return resultado;
	}
	
	/**
	 * Mtodo que recalcula o fitness (aptido) de um indviduo baseado. <BR>
	 * Necessrio quando h alterao em alguma cidade da rota.
	 */		
	public void recalculaFitness()
	{
		this.fitness = 0;
		calculaFitness();
	}

	/**
	 * Mtodo utilizado para clonar um cromossomo. Utilizado quando se usa o construtor sem parmetros. <BR> 
	 * Devem ser passados por parmetros todas as informaes do cromossomo que se deseja clonar
	 * @param totalCidades total de cidades que estaro na rota para o caixeiro viajar
	 * @param custos objeto do tipo Custos com os custos de deslocamento de todas as cidades
	 * @param cidadesCopiar cidades da rota que se deseja clonar
	 */		
	public void clonaCromossomo(int totalCidades, Custos custos, int cidadesCopiar[])
	{
		this.totalCidades = totalCidades;
		this.custos = custos;
		this.fitness = 0;
		individuo = new int[totalCidades-1];
		for (int i=0; i< totalCidades-1; i++)
			individuo[i] = cidadesCopiar[i];
	}

	/**
	 * Mtodo auxiliar utilizado para depurar o cdigo.
	 * @deprecated
	 * utilizado somente para depurao do cdigo.
	 */		
	public void mostraInformacoes()
	{
		for (int i=0; i< totalCidades-1; i++)
			System.out.println(individuo[i]);
		System.out.println("Fitness: " + fitness + "\n");		
	}	
	
	//  *********Acessores e modificadores ***********//
	
	public int getFitness() 
	{
		return this.fitness;
	}
	
	public double getChanceReproducao() 
	{
		return this.chanceReproducao;
	}

	public void setChanceReproducao(double chanceReproducao) 
	{
		this.chanceReproducao = chanceReproducao;
	}
	
	public int getTotalCidades() 
	{
		return this.totalCidades;
	}

	public Custos getCustos() 
	{
		return this.custos;
	}
	
	public int[] getIndividuo() 
	{
		return this.individuo;
	}	
	
}
