import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;

import javax.swing.*;

public class Principal extends JFrame implements ActionListener 
{
	private static final long serialVersionUID = 1L;
	private int totalCidades;
	private Custos custos;
	
	/** Variveis formulrio*/
	private JTextField txtCaminhoArquivo;
	private JTextField txtTamPopulacao;
	private JTextField txtTotalCiclos;
	private JButton btnAbrirArquivo;
	private JButton btnIniciar;
	private JTextArea memo;
	
	private Principal()
	{
		super("Caixeiro Viajante Genticos || Douglas / Reinaldo");
		inicializaComponentes();
		txtTotalCiclos.setText(txtTotalCiclos.getText().trim());
		txtTamPopulacao.setText(txtTamPopulacao.getText().trim());
	}
	
	public void iniciar()
	{
		memo.setText("");
		int ciclo = 0;
		Populacao p = new Populacao(Integer.parseInt(txtTamPopulacao.getText().trim()), totalCidades, custos);
		for (int i=0; i < Integer.parseInt(txtTotalCiclos.getText().trim()); i++)
		{
			ciclo = i+1;
			addMemoText("Melhor avaliado do ciclo "+ ciclo +" : \n\t" + 
						"Fitness: " + p.getMelhorAvaliado().getFitness() + "\n\tRota: " + p.getMelhorAvaliado().individuoToString() + "\n");
			p.geraNovaPopulacao();
		} 
	}
	
	private void inicializaComponentes()
	{
		Font fontePadrao = new Font("Tahoma", Font.BOLD, 14);
		
		txtCaminhoArquivo = new JTextField();
		txtCaminhoArquivo.setEditable(false);
		
		btnAbrirArquivo = new JButton("Abrir Arquivo");
		btnAbrirArquivo.addActionListener(this);
		
		JLabel lbArquivo = new JLabel("Caminho do Arquivo:");
		lbArquivo.setFont(fontePadrao);
		
		JPanel ptopo = new JPanel();
		ptopo.setLayout(new BorderLayout());
		ptopo.add(txtCaminhoArquivo, BorderLayout.CENTER);
		ptopo.add(btnAbrirArquivo, BorderLayout.EAST);
		ptopo.add(lbArquivo, BorderLayout.NORTH);
		
		JPanel pTopoSul = new JPanel(); //FlowLayout default
		
		JLabel lbPopulacao = new JLabel("Tamanho Populao: ");
		lbPopulacao.setFont(fontePadrao);
		
		JLabel lbCiclo = new JLabel("Total de Ciclos : ");
		lbCiclo.setFont(fontePadrao);
		
		txtTamPopulacao = new JTextField("20                   ");
		txtTotalCiclos = new JTextField("100                  ");
				
		btnIniciar = new JButton("Iniciar");
		btnIniciar.addActionListener(this);
		btnIniciar.setEnabled(false);
		
		pTopoSul.add(lbPopulacao);
		pTopoSul.add(txtTamPopulacao);
		pTopoSul.add(lbCiclo);
		pTopoSul.add(txtTotalCiclos);
		pTopoSul.add(btnIniciar);
		ptopo.add(pTopoSul, BorderLayout.SOUTH);
		
		JScrollPane scroll = new JScrollPane();
		memo = new JTextArea();
		scroll.setViewportView(memo);
		
		getContentPane().add(ptopo, BorderLayout.NORTH);
		getContentPane().add(scroll, BorderLayout.CENTER);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setMinimumSize(new Dimension(600, 400));
		pack();
		centralizarJanela(this);
		setVisible(true);
	}
	
	private void centralizarJanela(JFrame janela)
	{
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();   
        Dimension frameSize = getSize();   
        janela.setLocation(new Point((screenSize.width - frameSize.width) / 2,   
                (screenSize.height - frameSize.height) / 2));   
	}
	
	private boolean abrirArquivo()
	{
		JFileChooser openDialog = new JFileChooser();
		int resultado = openDialog.showOpenDialog(this);
		if (resultado == JFileChooser.CANCEL_OPTION)
			return false;
		
		File nomeArquivo = openDialog.getSelectedFile();
	
		if (nomeArquivo == null || nomeArquivo.getName().equals(""))
		{
			JOptionPane.showMessageDialog(this, "Arquivo invlido", "Erro", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		
		txtCaminhoArquivo.setText(nomeArquivo.getAbsolutePath());
		try 
		{
			ParserArquivo p = new ParserArquivo(new BufferedReader(new FileReader(nomeArquivo)));
			p.parser();
			this.totalCidades = p.getTotalCidades();
			this.custos = new Custos(p.getCustos());
		} 
		catch (Exception e) 
		{
			JOptionPane.showMessageDialog(this, e.toString(), "Erro", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}
	
	public void actionPerformed(ActionEvent event) 
	{
		if (event.getSource() == btnAbrirArquivo)
		{
			memo.setText(null);
			if (abrirArquivo()) 
			{
				memo.setText("Arquivo vlido! \nTotal de Cidades: " + totalCidades);
				btnIniciar.setEnabled(true);
			}
		}
		else
		if (event.getSource() == btnIniciar)
		{
			addMemoText("\nIniciando processo .....");
			iniciar();
		}
	}
	
	private void addMemoText(String text)
	{
		memo.setText(memo.getText() + "\n" + text);
	}
	
	public static void main(String[] args) 
	{
		try 
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} 
		catch (Exception e)	
		{
			JOptionPane.showMessageDialog(null, e.toString(), "Erro", JOptionPane.ERROR_MESSAGE);
		}
		
		new Principal();
	}

}
