import java.util.Random;
import java.util.Vector;

class Novos
{
	public int cidade;
	public int indice;
}

/** Classe responsvel pelas reprodues dos cromossomos: <BR>
 * Assexuada <BR>
 * Mutao<BR>
 * Sexuada<BR>
 * <BR><BR>
 * Trabalho de Algoritmos Genticos <BR> 
 * Disciplina de Inteligncia Artificial <BR>
 * Sistemas de Informao - 01/2009 <BR>
 * Professor: Guilherme Dhein <BR>
 * <BR>
 * @author Douglas Pasqualin
 * @author Reinaldo Dornelles
 *
 */
public class Reproducao 
{
	private int totalCidades;
	private Vector<Novos> novosA;
	private Vector<Novos> novosB;
	
	public Reproducao(int totalCidades)
	{
		this.totalCidades = totalCidades;
		this.novosA = new Vector<Novos>();
		this.novosB = new Vector<Novos>();
	}
	
	/**
	 * Reproduo Assexuada
	 * @param fonte Cromossomo que se deseja reproduzir
	 * @return novo Cromossomo exatamente igual ao passado por parmetro
	 */
	public Cromossomo reproducaoAssexuada(Cromossomo fonte)
	{
		Cromossomo novo = new Cromossomo();
		novo.clonaCromossomo(fonte.getTotalCidades(), fonte.getCustos(), fonte.getIndividuo());
		novo.recalculaFitness();
		return novo;
	}

	/**
	 * Reproduo por Mutao
	 * @param fonte Cromossomo que se deseja reproduzir
	 * @return novo Cromossomo com apenas duas cidades modificadas na rota
	 */	
	public Cromossomo mutacao(Cromossomo fonte)
	{
		int i1, i2;
		Cromossomo novo = new Cromossomo();
		novo.clonaCromossomo(fonte.getTotalCidades(), fonte.getCustos(), fonte.getIndividuo());
		Random random = new Random();
		i1 = random.nextInt(totalCidades-1);
		do 
		{
			i2 = random.nextInt(totalCidades-1);	
		} while (i1 == i2);
		int aux = novo.individuo[i1];
		novo.individuo[i1] = novo.individuo[i2];
		novo.individuo[i2] = aux;
		novo.recalculaFitness();
		return novo;
	}

	/**
	 * Reproduo Sexuada
	 * @param fontes Cromossomos que se deseja reproduzir
	 * @return vetor com dois novos cromossomos com bastante variaes nas cidades da rota
	 */		
	public Cromossomo[] reproducaoSexuada(Cromossomo[] fontes)
	{
		int i1, i2;
		limpaVectors();
		Cromossomo[] novo = new Cromossomo[2];
		novo[0] = new Cromossomo();
		novo[0].clonaCromossomo(fontes[0].getTotalCidades(), fontes[0].getCustos(), fontes[0].getIndividuo());
		novo[1] = new Cromossomo();
		novo[1].clonaCromossomo(fontes[1].getTotalCidades(), fontes[1].getCustos(), fontes[1].getIndividuo());
		Random random = new Random();
		
		//Escolhe aleatoriamente onde vo ser feitos os cortes para reproduo
		do
		{
			i1 = random.nextInt(totalCidades / 2);
		}
		while (i1 < 1);
		do 
		{
			i2 = random.nextInt(totalCidades-1);	
		} while (i1+1 >= i2); //Garante pelo menos 1 intervalo entre os cortes

		//Copia trecho que no ser mudado
		for (int i=i1; i <i2; i++)
		{
			novo[0].individuo[i] = fontes[1].individuo[i];
			novo[1].individuo[i] = fontes[0].individuo[i];
			//Guarda os novos que foram inseridos em cada cromossomo novo
			//Necessrio para o PMX Crossover
			adicionaEmNovasCidadesA(fontes[1].individuo[i], i);
			adicionaEmNovasCidadesB(fontes[0].individuo[i], i);
		}
		
		//Verifica necessidade de aplicar PMX Crossover
		//Trecho 1:
		int aux = 0;
		do
		{
			for (int i=0; i < i1; i++)
			{
				aux = cidadeDuplicada(novo[0].individuo[i], novosA);
				/**Se aux for maior que zero  sinal que a cidade que to testando
				 * j foi inserida na reproduo, e em aux eu tenho o indice onde ela foi 
				 * inserida. S preciso trocar com o que tinha nessa posio no vetor original!
				 */
				if (aux > 0)
					novo[0].individuo[i] = fontes[0].individuo[aux];

				//Mesmo procedimento para o segundo indivduo
				aux = cidadeDuplicada(novo[1].individuo[i], novosB);
				if (aux > 0)
					novo[1].individuo[i] = fontes[1].individuo[aux];			
			}

			//Trecho 2
			for (int i=i2; i < totalCidades-1; i++)
			{
				aux = cidadeDuplicada(novo[0].individuo[i], novosA);
				if (aux > 0)
					novo[0].individuo[i] = fontes[0].individuo[aux];

				//Mesmo procedimento para o segundo indivduo
				aux = cidadeDuplicada(novo[1].individuo[i], novosB);
				if (aux > 0)
					novo[1].individuo[i] = fontes[1].individuo[aux];			
			}
		//Repete at que no tenha nenhum conflito de cidades!
		} while (existeRepetido(novo[0].getIndividuo()) ||existeRepetido(novo[1].getIndividuo())); 
		
	
		//Recalcular Fitness
		novo[0].recalculaFitness();
		novo[1].recalculaFitness();
		
		return novo;
	}
	
	private void limpaVectors()
	{
		novosA.removeAllElements();
		novosB.removeAllElements();
	}
	
	private int cidadeDuplicada(int cidade, Vector<Novos> cidades)
	{
		for (int i=0; i < cidades.size(); i++)
			if (cidades.get(i).cidade == cidade)
				return cidades.get(i).indice;

		return -1;
	}
	
	private void adicionaEmNovasCidadesA(int cidade, int indice)
	{
		Novos novasCidades = new Novos();
		novasCidades.cidade = cidade;
		novasCidades.indice = indice;
		novosA.add(novasCidades);
	}
	
	private void adicionaEmNovasCidadesB(int cidade, int indice)
	{
		Novos novasCidades = new Novos();
		novasCidades.cidade = cidade;
		novasCidades.indice = indice;
		novosB.add(novasCidades);
	}	
	
	/**
	 * Metdo que varre as cidades da rota para verificao se existem cidades duplicadas na rota
	 * @param cidades vetor de cidades que ser testado
	 * @return <b>true</b> se existirem repetidos e <b>false</b> caso no exista
	 */
	private boolean existeRepetido(int[] cidades)
	{
		for (int i=0; i < cidades.length; i++)
			for (int j=0; j < cidades.length; j++)
				if (i != j && cidades[i] == cidades[j])
					return true;
		return false;
	}
	
}
